@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from message import Message
 from gestionatr.input.messages.C2 import Direccion
-from gestionatr.defs import TARIFES_SEMPRE_MAX
+from gestionatr.defs import TARIFES_SEMPRE_MAX, TARIFES_TD
 from datetime import datetime, date
 from gestionatr.utils import repartir_consums_entre_lectures
 
@@ -10,6 +10,7 @@ MAGNITUDS_OCSUM = {
     'AE': 'A',
     'AS': 'S',
     'R1': 'R',
+    'R4': 'RC',
     'PM': 'M',
     'EP': 'EP'
 }
@@ -38,8 +39,22 @@ PERIODE_OCSUM = {
     '81': 'P1',  # P1 Tarifa 007
     '82': 'P2',  # P2 Tarifa 007
     '83': 'P3',  # P3 Tarifa 007
+    '90': 'P1',  # P1 de peaje de acceso 2.0TD.
+    '91': 'P1',  # P1 de peaje de acceso 2.0TD.
+    '92': 'P2',  # P2 de peaje de acceso 2.0TD.
+    '93': 'P3',  # P3 de peaje de acceso 2.0TD.
+    'A0': 'P1',  # P1 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
+    'A1': 'P1',  # P1 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
+    'A2': 'P2',  # P2 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
+    'A3': 'P3',  # P3 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
+    'A4': 'P4',  # P4 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
+    'A5': 'P5',  # P5 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
+    'A6': 'P6',  # P6 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
+
 }
 
+PERIODES_NO_TD = [x for x in PERIODE_OCSUM.keys() if x[0] not in ['A', '9']]
+
 CODIS_REG_REFACT = {
     'RGT42011': '40',
     'RGT12012': '41',
@@ -72,6 +87,65 @@ CODIS_AUTOCONSUM = {
 # Totalitzadors a ignorar
 SKIP_TOTALITZADORS = ('00', '60')
 
+PERIODES_PER_TARIFA = {
+    '018': {
+        'A': 3,
+        'S': 3,
+        'R': 3,
+        'RC': 3,
+        'M': 2,
+        'EP': 2,
+    },
+    '019': {
+        'A': 6,
+        'S': 6,
+        'R': 6,
+        'RC': 6,
+        'M': 6,
+        'EP': 6,
+    },
+    '021': {
+        'A': 6,
+        'S': 6,
+        'R': 6,
+        'RC': 6,
+        'M': 6,
+        'EP': 6,
+    },
+    '022': {
+        'A': 6,
+        'S': 6,
+        'R': 6,
+        'RC': 6,
+        'M': 6,
+        'EP': 6,
+    },
+    '023': {
+        'A': 6,
+        'S': 6,
+        'R': 6,
+        'RC': 6,
+        'M': 6,
+        'EP': 6,
+    },
+    '024': {
+        'A': 6,
+        'S': 6,
+        'R': 6,
+        'RC': 6,
+        'M': 6,
+        'EP': 6,
+    },
+    '025': {
+        'A': 6,
+        'S': 6,
+        'R': 6,
+        'RC': 6,
+        'M': 6,
+        'EP': 6,
+    },
+}
+
 
 class F1(Message):
     """Clase que implementa F1."""
@@ -232,6 +306,21 @@ class DatosGeneralesATR(DatosGenerales):
             return self._datos_factura_atr.FechaBOE.text.strip()
         return None
 
+    @property
+    def tipo_autoconsumo(self):
+        if hasattr(self._datos_factura_atr, 'TipoAutoconsumo'):
+            return self._datos_factura_atr.TipoAutoconsumo.text.strip()
+
+    @property
+    def cau(self):
+        if hasattr(self._datos_factura_atr, 'CAU'):
+            return self._datos_factura_atr.CAU.text.strip()
+
+    @property
+    def duracion_inf_anio(self):
+        if hasattr(self._datos_factura_atr, 'DuracionInfAnio'):
+            return self._datos_factura_atr.DuracionInfAnio.text.strip()
+
     @property
     def tarifa_atr_fact(self):
         if hasattr(self._datos_factura_atr, 'TarifaATRFact'):
@@ -310,6 +399,12 @@ class DatosGeneralesATR(DatosGenerales):
             return int(self._periodo.NumeroDias.text.strip())
         return None
 
+    @property
+    def tipo_pm(self):
+        if hasattr(self._periodo, 'TipoPM'):
+            return int(self._periodo.TipoPM.text.strip())
+        return None
+
 
 class DatosGeneralesOtras(DatosGenerales):
 
@@ -380,12 +475,26 @@ class Factura(object):
         return data
 
     def get_coeficient_repartiment(self):
+        try:
+            if self.factura.Autoconsumo and self.factura.Autoconsumo.InstalacionGenAutoconsumo:
+                beta_list = []
+                for instalacio in self.factura.Autoconsumo.InstalacionGenAutoconsumo:
+                    if instalacio.EnergiaNetaGen:
+                        for terme in instalacio.EnergiaNetaGen.TerminoEnergiaNetaGen:
+                            for periode in terme.Periodo:
+                                beta_list.append(float(periode.Beta.text))
+                return list(set(beta_list))
+        except AttributeError:
+            # We might not have any "Coeficient de repartiment" in EnergiaNetaGen
+            pass
+
+    def get_coeficient_repartiment_from_cr(self):
         try:
             for concepte in self.conceptos_repercutibles:
                 if concepte.concepto_repercutible == '82':
                     return concepte.unidades
         except AttributeError:
-            # We might not have any "Coeficient de repartiment"
+            # We might not have any "Coeficient de repartiment" in conceptos repercutibles
             pass
 
     def get_info_conceptes_repercutibles(self):
@@ -415,16 +524,37 @@ class Factura(object):
 
     def get_linies_factura_by_type(self):
         res = {}
+        res_to_join = {}
 
+        to_join = {
+            'potencia_cargos': 'potencia',
+            'energia_cargos': 'energia'
+        }
         for type, method in self.GETTERS_LINEAS_FACTURA:
             lines, sub_total = method()
 
             if lines:
-                res.setdefault(type, {'total': 0.0, 'lines': []})
+                if type in to_join.keys():
+                    aux_res = res_to_join
+                else:
+                    aux_res = res
 
-                res[type]['lines'] += lines
-                new_total = res[type]['total'] + sub_total
-                res[type]['total'] = round(new_total, 2)
+                aux_res.setdefault(type, {'total': 0.0, 'lines': []})
+
+                aux_res[type]['lines'] += lines
+                new_total = aux_res[type]['total'] + sub_total
+                aux_res[type]['total'] = round(new_total, 2)
+
+        for tipus, info in res_to_join.items():
+            tipus_join = to_join.get(tipus)
+            if not res.get(tipus_join):
+                continue
+            base = 1.0
+            res[tipus_join]['total'] += info['total']
+            for l in info['lines']:
+                for l2 in res[tipus_join]['lines']:
+                    if l2.nombre == l.nombre and l2.cantidad == l.cantidad:
+                        l2.precio += round(l.precio*base, 9)
 
         return res
 
@@ -459,6 +589,7 @@ class Periodo(object):
         self._name = name
         self.fecha_desde = fecha_desde
         self.fecha_hasta = fecha_hasta
+        self._precio = 0.0
 
     @property
     def nombre(self):
@@ -466,11 +597,17 @@ class Periodo(object):
 
     @property
     def precio(self):
-        if self.NOMBRE_PRECIO:
+        if self._precio:
+            return self._precio
+        elif self.NOMBRE_PRECIO:
             if hasattr(self.periodo, self.NOMBRE_PRECIO):
                 return float(getattr(self.periodo, self.NOMBRE_PRECIO).text.strip())
         return None
 
+    @precio.setter
+    def precio(self, value):
+        self._precio = value
+
     @property
     def cantidad(self):
         if self.NOMBRE_CANTIDAD:
@@ -509,6 +646,12 @@ class PeriodoPotencia(Periodo):
             return int(self.periodo.PotenciaAFacturar.text.strip())
         return None
 
+    @property
+    def recargo_inf_anio(self):
+        if hasattr(self.periodo, 'RecargoInfAnio'):
+            return int(self.periodo.RecargoInfAnio.text.strip())
+        return None
+
 
 class Termino(object):
 
@@ -699,11 +842,158 @@ class PeriodoEnergiaReactiva(Periodo):
         return None
 
 
+class PeriodoEnergiaCapacitiva(Periodo):
+
+    NOMBRE_PRECIO = 'PrecioEnergiaCapacitiva'
+    NOMBRE_CANTIDAD = 'ValorEnergiaCapacitiva'
+
+    @property
+    def valor_energia_capacitiva(self):
+        if hasattr(self.periodo, 'ValorEnergiaCapacitiva'):
+            return float(self.periodo.ValorEnergiaCapacitiva.text.strip())
+        return None
+
+
+class PeriodoEnergiaNetaGen(Periodo):
+
+    NOMBRE_CANTIDAD = 'ValorEnergiaNetaGen'
+
+    @property
+    def valor_energia_neta_gen(self):
+        if hasattr(self.periodo, 'ValorEnergiaNetaGen'):
+            return float(self.periodo.ValorEnergiaNetaGen.text.strip())
+        return None
+
+    @property
+    def beta(self):
+        if hasattr(self.periodo, 'Beta'):
+            return float(self.periodo.Beta.text.strip())
+        return None
+
+    @property
+    def relacion_generacion(self):
+        if hasattr(self.periodo, 'RelacionGeneracion'):
+            return float(self.periodo.RelacionGeneracion.text.strip())
+        return None
+
+
+class PeriodoEnergiaAutoconsumida(Periodo):
+
+    NOMBRE_CANTIDAD = 'ValorEnergiaAutoconsumida'
+    NOMBRE_PRECIO = "PagoTDA"
+
+    @property
+    def valor_energia_autoconsumida(self):
+        if hasattr(self.periodo, 'ValorEnergiaAutoconsumida'):
+            return float(self.periodo.ValorEnergiaAutoconsumida.text.strip())
+        return None
+
+    @property
+    def pago_tda(self):
+        if hasattr(self.periodo, 'PagoTDA'):
+            return float(self.periodo.PagoTDA.text.strip())
+        return None
+
+
+class PeriodoEnergiaExcedentaria(Periodo):
+
+    NOMBRE_CANTIDAD = 'ValorEnergiaExcedentaria'
+
+    @property
+    def valor_energia_excedentaria(self):
+        if hasattr(self.periodo, 'ValorEnergiaExcedentaria'):
+            return float(self.periodo.ValorEnergiaExcedentaria.text.strip())
+        return None
+
+
+class PeriodoCargo(Periodo):
+
+    NOMBRE_PRECIO = "PrecioCargo"
+
+    @property
+    def energia(self):
+        if hasattr(self.periodo, 'Energia'):
+            return float(self.periodo.Energia.text.strip())
+        return None
+
+    @property
+    def potencia(self):
+        if hasattr(self.periodo, 'Potencia'):
+            return float(self.periodo.Potencia.text.strip())
+        return None
+
+    @property
+    def precio_cargo(self):
+        if hasattr(self.periodo, 'PrecioCargo'):
+            return float(self.periodo.PrecioCargo.text.strip())
+        return None
+
+    def es_facturable(self):
+        return self.precio_cargo or self.potencia or self.energia
+
+    @property
+    def cantidad(self):
+        return self.potencia or self.energia or 0.0
+
 class TerminoEnergiaReactiva(TerminoEnergiaActiva):
 
     PERIODO_TYPE = PeriodoEnergiaReactiva
 
 
+class TerminoEnergiaCapacitiva(TerminoEnergiaActiva):
+
+    PERIODO_TYPE = PeriodoEnergiaCapacitiva
+
+
+class TerminoEnergiaNetaGen(TerminoEnergiaActiva):
+
+    PERIODO_TYPE = PeriodoEnergiaNetaGen
+
+
+class TerminoEnergiaAutoconsumida(TerminoEnergiaActiva):
+
+    PERIODO_TYPE = PeriodoEnergiaAutoconsumida
+
+
+class TerminoEnergiaExcedentaria(TerminoEnergiaActiva):
+
+    PERIODO_TYPE = PeriodoEnergiaExcedentaria
+
+
+class TerminoCargo(TerminoEnergiaActiva):
+
+    PERIODO_TYPE = PeriodoCargo
+
+
+class InstalacionGenAutoconsumo(object):
+    def __init__(self, data):
+        self.instalacion_gen_autoconsumo = data
+
+    @property
+    def tipo_instalacion(self):
+        if hasattr(self.instalacion_gen_autoconsumo, 'TipoInstalacion'):
+            return self.instalacion_gen_autoconsumo.TipoInstalacion.text.strip()
+        return None
+
+    @property
+    def exento_cargos(self):
+        if hasattr(self.instalacion_gen_autoconsumo, 'ExentoCargos'):
+            return self.instalacion_gen_autoconsumo.ExentoCargos.text.strip()
+        return None
+
+    @property
+    def energia_neta_gen(self):
+        if hasattr(self.instalacion_gen_autoconsumo, 'EnergiaNetaGen'):
+            return EnergiaNetaGen(self.instalacion_gen_autoconsumo.EnergiaNetaGen)
+        return None
+
+    @property
+    def energia_autoconsumida(self):
+        if hasattr(self.instalacion_gen_autoconsumo, 'EnergiaAutoconsumida'):
+            return EnergiaAutoconsumida(self.instalacion_gen_autoconsumo.EnergiaAutoconsumida)
+        return None
+
+
 class EnergiaReactiva(object):
 
     def __init__(self, data):
@@ -724,6 +1014,152 @@ class EnergiaReactiva(object):
         return None
 
 
+class EnergiaCapacitiva(object):
+
+    def __init__(self, data):
+        self.energia_capacitiva = data
+
+    @property
+    def terminos_energia_capacitiva(self):
+        data = []
+        if hasattr(self.energia_capacitiva, 'TerminoEnergiaCapacitiva'):
+            for d in self.energia_capacitiva.TerminoEnergiaCapacitiva:
+                data.append(TerminoEnergiaCapacitiva(d))
+        return data
+
+    @property
+    def importe_total(self):
+        if hasattr(self.energia_capacitiva, 'ImporteTotalEnergiaCapcitiva'):
+            return float(self.energia_capacitiva.ImporteTotalEnergiaCapcitiva.text.strip())
+        return None
+
+
+class EnergiaNetaGen(object):
+
+    def __init__(self, data):
+        self.energia_neta_gen = data
+
+    @property
+    def terminos_energia_neta_gen(self):
+        data = []
+        if hasattr(self.energia_neta_gen, 'TerminoEnergiaNetaGen'):
+            for d in self.energia_neta_gen.TerminoEnergiaNetaGen:
+                data.append(TerminoEnergiaNetaGen(d))
+        return data
+
+    @property
+    def importe_total(self):
+        if hasattr(self.energia_neta_gen, 'TotalEnergiaNetaGenBeta'):
+            return float(self.energia_neta_gen.TotalEnergiaNetaGenBeta.text.strip())
+        return None
+
+
+class EnergiaAutoconsumida(object):
+
+    def __init__(self, data):
+        self.energia_autoconsumida = data
+
+    @property
+    def terminos_energia_autoconsumida(self):
+        data = []
+        if hasattr(self.energia_autoconsumida, 'TerminoEnergiaAutoconsumida'):
+            for d in self.energia_autoconsumida.TerminoEnergiaAutoconsumida:
+                data.append(TerminoEnergiaAutoconsumida(d))
+        return data
+
+    @property
+    def importe_total(self):
+        if hasattr(self.energia_autoconsumida, 'ImporteTotalEnergiaActivaAutoconsumida'):
+            return float(self.energia_autoconsumida.ImporteTotalEnergiaActivaAutoconsumida.text.strip())
+        return None
+
+
+class EnergiaExcedentaria(object):
+
+    def __init__(self, data):
+        self.energia_excedentaria = data
+
+    @property
+    def terminos_energia_excedentaria(self):
+        data = []
+        if hasattr(self.energia_excedentaria, 'TerminoEnergiaExcedentaria'):
+            for d in self.energia_excedentaria.TerminoEnergiaExcedentaria:
+                data.append(TerminoEnergiaExcedentaria(d))
+        return data
+
+    @property
+    def importe_total(self):
+        if hasattr(self.energia_excedentaria, 'ValorTotalEnergiaExcedentaria'):
+            return float(self.energia_excedentaria.ValorTotalEnergiaExcedentaria.text.strip())
+        return None
+
+
+class Cargo(object):
+
+    def __init__(self, data):
+        self.cargo = data
+
+    @property
+    def tipo_cargo(self):
+        if hasattr(self.cargo, 'TipoCargo'):
+            return float(self.cargo.TipoCargo.text.strip())
+        return None
+
+    @property
+    def termino_cargo(self):
+        data = []
+        if hasattr(self.cargo, 'TerminoCargo'):
+            for d in self.cargo.TerminoCargo:
+                data.append(TerminoCargo(d))
+        return data
+
+    @property
+    def importe_total(self):
+        if hasattr(self.cargo, 'TotalImporteTipoCargo'):
+            return float(self.cargo.TotalImporteTipoCargo.text.strip())
+        return None
+
+
+class Autoconsumo(object):
+
+    def __init__(self, data):
+        self.autoconsumo = data
+
+    @property
+    def instalacion_gen_autoconsumo(self):
+        data = []
+        if hasattr(self.autoconsumo, 'InstalacionGenAutoconsumo'):
+            for d in self.autoconsumo.InstalacionGenAutoconsumo:
+                data.append(InstalacionGenAutoconsumo(d))
+        return data
+
+    @property
+    def energia_excedentaria(self):
+        if hasattr(self.autoconsumo, 'EnergiaExcedentaria'):
+            return EnergiaExcedentaria(self.autoconsumo.EnergiaExcedentaria)
+        return None
+
+
+class Cargos(object):
+
+    def __init__(self, data):
+        self.cargos = data
+
+    @property
+    def cargo(self):
+        data = []
+        if hasattr(self.cargos, 'Cargo'):
+            for d in self.cargos.Cargo:
+                data.append(Cargo(d))
+        return data
+
+    @property
+    def total_cargos(self):
+        if hasattr(self.cargos, 'TotalImporteCargos'):
+            return float(self.cargos.TotalImporteCargos.text.strip())
+        return None
+
+
 class Impuesto(object):
 
     def __init__(self, data):
@@ -991,7 +1427,12 @@ class Integrador(object):
 
     @property
     def periode(self):
-        return PERIODE_OCSUM.get(self.codigo_periodo, None)
+        p = PERIODE_OCSUM.get(self.codigo_periodo, None)
+        if not p and self.codigo_periodo in PERIODE_OCSUM.values():
+            return self.codigo_periodo
+        if self.codigo_periodo == '93' and p == 'P3' and self.magnitud in ('PM', 'EP'):
+            return "P2"
+        return p
 
     @property
     def gir_comptador(self):
@@ -1089,7 +1530,7 @@ class ModeloAparato(object):
     def get_dates_inici_i_final(self):
         data_inici = ''
         data_final = ''
-        for lect in self.get_lectures():
+        for lect in self.get_lectures(force_no_transforma_no_td_a_td=True):
             data_in_compt = datetime.strptime(
                 lect.lectura_desde.fecha, '%Y-%m-%d'
             )
@@ -1104,7 +1545,7 @@ class ModeloAparato(object):
 
         return data_inici, data_final
 
-    def get_lectures(self, tipus=None):
+    def get_lectures(self, tipus=None, force_no_transforma_no_td_a_td=False):
         """Retorna totes les lectures en una llista de Lectura"""
         lectures = []
         try:
@@ -1127,6 +1568,9 @@ class ModeloAparato(object):
             # Si nomes ens envien el P0 de excedents pero ens cobren varis periodes
             # creem una lectura e P2 AS ficticies a 0 (puta FENOSA)
             lectures.extend(self.factura.get_fake_AS_p2_lectures())
+
+        if not force_no_transforma_no_td_a_td and self.factura:
+            lectures = self.factura.transforma_no_td_a_td(lectures, tipus=tipus)
         return lectures
 
     def get_lectures_activa(self):
@@ -1142,7 +1586,7 @@ class ModeloAparato(object):
         return self.get_lectures(['R'])
 
     def get_lectures_energia(self):
-        return self.get_lectures(['A', 'S', 'R'])
+        return self.get_lectures(['A', 'S', 'R', 'RC'])
 
     def get_lectures_maximetre(self):
         return self.get_lectures(['M'])
@@ -1233,18 +1677,36 @@ class FacturaATR(Factura):
 
         self.GETTERS_LINEAS_FACTURA += [
             ('potencia', self.get_info_potencia),
+            ('potencia_cargos', self.get_info_potencia_cargos),
             ('exces_potencia', self.get_info_exces),
             ('energia', self.get_info_activa),
+            ('energia_cargos', self.get_info_energia_cargos),
             ('reactiva', self.get_info_reactiva),
             ('lloguer', self.get_info_lloguer),
+            ('generacio', self.get_info_generacio),
+            ('generacio_neta', self.get_info_generacio_neta),
+            ('autoconsum', self.get_info_autoconsumo),
         ]
 
     def te_autoconsum(self):
+        if self.datos_factura and self.datos_factura.tipo_autoconsumo and self.datos_factura.tipo_autoconsumo not in ['00', '01', '2A', '2B', '2G']:
+            return True
         for concepte in self.conceptos_repercutibles:
             if concepte.concepto_repercutible in CODIS_AUTOCONSUM.keys():
                 return True
         return False
 
+    def te_lectures_pre_td_amb_tarifa_td(self):
+        if self.datos_factura.tarifa_atr_fact not in TARIFES_TD:
+            return False
+
+        for c in self.get_comptadors():
+            for l in c.get_lectures(force_no_transforma_no_td_a_td=True):
+                if l.codigo_periodo in PERIODES_NO_TD:
+                    return True
+
+        return False
+
     @property
     def potencia(self):
         if hasattr(self.factura, 'Potencia'):
@@ -1269,6 +1731,24 @@ class FacturaATR(Factura):
             return EnergiaReactiva(self.factura.EnergiaReactiva)
         return None
 
+    @property
+    def energia_capacitiva(self):
+        if hasattr(self.factura, 'EnergiaCapacitiva'):
+            return EnergiaCapacitiva(self.factura.EnergiaCapacitiva)
+        return None
+
+    @property
+    def autoconsumo(self):
+        if hasattr(self.factura, 'Autoconsumo'):
+            return Autoconsumo(self.factura.Autoconsumo)
+        return None
+
+    @property
+    def cargos(self):
+        if hasattr(self.factura, 'Cargos'):
+            return Cargos(self.factura.Cargos)
+        return None
+
     @property
     def impuesto_electrico(self):
         if hasattr(self.factura, 'ImpuestoElectrico'):
@@ -1289,6 +1769,53 @@ class FacturaATR(Factura):
                 data.append(Medida(d))
         return data
 
+    def transforma_no_td_a_td(self, lectures, tipus=None):
+        if self.datos_factura.tarifa_atr_fact not in TARIFES_TD:
+            return lectures
+
+        if not self.te_lectures_pre_td_amb_tarifa_td():
+            return lectures
+
+        if not tipus:
+            tipus_lectures = ['A', 'S', 'R', 'RC' 'M', 'EP']
+        else:
+            tipus_lectures = tipus
+
+        res = []
+        for t in tipus_lectures:
+            lectures_amb_ajustos = self.get_lectures_amb_periodes_td(lectures, t)
+            res.extend(lectures_amb_ajustos)
+        return res
+
+    def get_lectures_amb_periodes_td(self, lectures, tipus):
+        res = []
+        tarifa_atr = self.datos_factura.tarifa_atr_fact
+        nperiodes_td = PERIODES_PER_TARIFA.get(tarifa_atr, {}).get(tipus, None)
+        if not nperiodes_td:
+            return [x for x in lectures if x.tipus == tipus]
+
+        lectures_per_periode = {}
+        for periode in range(1, nperiodes_td+1):
+            pname = "P"+str(periode)
+            lectures_per_periode[pname] = []
+
+        base_lectura = None
+        for l in lectures:
+            if l.tipus == tipus:
+                lectures_per_periode[l.periode].append(l)
+                base_lectura = l
+
+        if not base_lectura:
+            return [x for x in lectures if x.tipus == tipus]
+
+        for periode in lectures_per_periode:
+            if not lectures_per_periode.get(periode) and base_lectura:
+                aux = self.get_fake_pX_lectura(tipus, periode, base_lectura)
+                res.append(aux)
+            else:
+                res.extend(lectures_per_periode.get(periode))
+        return res
+
     def get_consum_facturat(self, tipus, periode=None):
         if tipus not in ['A', 'S']:
             return None
@@ -1305,13 +1832,46 @@ class FacturaATR(Factura):
 
         if tipus == 'S':
             res = []
-            for concepte in self.conceptos_repercutibles:
-                if concepte.concepto_repercutible[0] == '7' and (not periode or concepte.concepto_repercutible[1] == periode[1]):
-                    res.append(concepte.unidades)
+            if self.autoconsumo and self.autoconsumo.energia_excedentaria:
+                for terme in self.autoconsumo.energia_excedentaria.terminos_energia_excedentaria:
+                    for periode_ex in terme.periodos:
+                        if periode_ex.nombre == periode or not periode:
+                            res.append(periode_ex.cantidad)
+            else:
+                for concepte in self.conceptos_repercutibles:
+                    if concepte.concepto_repercutible[0] == '7' and (not periode or concepte.concepto_repercutible[1] == periode[1]):
+                        res.append(concepte.unidades)
             return res
 
         return None
 
+    def get_consums_reactiva_capacitiva_a_facturar(self):
+        res = {}
+        if not self.energia_capacitiva:
+            return {}
+        if not self.energia_capacitiva.terminos_energia_capacitiva:
+            return {}
+        for capacitiva in self.energia_capacitiva.terminos_energia_capacitiva:
+            for periode in capacitiva.periodos:
+                if periode.nombre not in res:
+                    res[periode.nombre] = 0.0
+                res[periode.nombre] += periode.cantidad
+        # El metode esta preparat per si ens envien 6 periodes, 5 amb valor 0 i l'ultim (el p6) amb valor ple
+        # Pero com que no se si faran aixo o nomes m'enviaran el P6, fem aquesta mandanga:
+        if len(res) == 1 and "P1" in res:
+            res['P6'] = res['P1']
+            del res['P1']
+        return res
+
+    def get_consums_autoconsumida_a_facturar(self):
+        res = {}
+        info, total = self.get_info_autoconsumo()
+        for periode in info:
+            if periode.nombre not in res:
+                res[periode.nombre] = 0.0
+            res[periode.nombre] += periode.cantidad
+        return res
+
     def get_lectures_amb_ajust_autoconsum(self, tipus='S', ajust_balancejat=True, motiu_ajust="98"):
         return self.get_lectures_amb_ajust_quadrat_amb_consum(tipus=tipus, ajust_balancejat=ajust_balancejat, motiu_ajust=motiu_ajust)
 
@@ -1326,10 +1886,15 @@ class FacturaATR(Factura):
                     if periode_activa.nombre not in res:
                         res[periode_activa.nombre] = True
 
-        else:  # if tipus == 'S':
+        elif tipus == 'S':
             for concepte in self.conceptos_repercutibles:
                 if concepte.concepto_repercutible[0] == '7' and concepte.concepto_repercutible not in res:
                     res[concepte.concepto_repercutible] = True
+            if self.autoconsumo and self.autoconsumo.energia_excedentaria:
+                for excedent in self.autoconsumo.energia_excedentaria.terminos_energia_excedentaria:
+                    for periode in excedent.periodos:
+                        if periode.nombre not in res:
+                            res[periode.nombre] = True
 
         return len(res.keys()) * 2 == nperiodes_lectures
 
@@ -1389,6 +1954,39 @@ class FacturaATR(Factura):
                     res.append(new_integrador)
         return res
 
+    def get_fake_pX_lectura(self, tipus, periode, base_info):
+        l1 = Lectura(None)
+        l1.fecha = base_info.lectura_desde.fecha
+        l1.lectura = 0
+        l1.procedencia = base_info.lectura_desde.procedencia
+        l2 = Lectura(None)
+        l2.fecha = base_info.lectura_hasta.fecha
+        l2.lectura = 0
+        l2.procedencia = base_info.lectura_hasta.procedencia
+
+        new_integrador = Integrador(None)
+        new_integrador.magnitud = {v: k for k, v in MAGNITUDS_OCSUM.iteritems()}.get(tipus, tipus)
+        new_integrador.numero_ruedas_enteras = base_info.numero_ruedas_enteras
+        new_integrador.codigo_periodo = periode
+        new_integrador.lectura_desde = l1
+        new_integrador.lectura_hasta = l2
+
+        # consums_desitjats = self.get_consum_facturat(tipus=tipus, periode=periode)
+        # if consums_desitjats and len(consums_desitjats) > 1:
+        #     raise Exception("No es poden calcular els ajustos de les lectures perque hi ha varis consums per 1 periode")
+        # elif consums_desitjats:
+        #     consum = consums_desitjats[0]
+        #     if not new_integrador.ajuste:
+        #         new_integrador.ajuste = Ajuste(None)
+        #     new_integrador.ajuste = Ajuste(new_integrador.ajuste.ajuste)
+        #     old_ajust = new_integrador.ajuste and new_integrador.ajuste.ajuste_por_integrador or 0.0
+        #     new_val = consum - (l2.lectura - l1.lectura + old_ajust)
+        #     if new_val != (l2.lectura - l1.lectura + old_ajust):
+        #         new_integrador.ajuste.ajuste_por_integrador = consum - (l2.lectura - l1.lectura)
+        #         new_integrador.ajuste.codigo_motivo = "99"
+
+        return new_integrador
+
     def get_fake_AS_p2_lectures(self):
         res = []
         comptador_amb_lectures = None
@@ -1500,6 +2098,26 @@ class FacturaATR(Factura):
             pass
         return periodes, total
 
+    def get_info_potencia_cargos(self):
+        return self.get_info_cargos(tipo_cargo=1)
+
+    def get_info_energia_cargos(self):
+        return self.get_info_cargos(tipo_cargo=2)
+
+    def get_info_cargos(self, tipo_cargo):
+        periodes = []
+        total = 0
+        try:
+            if self.cargos and self.cargos.cargo:
+                for cargo_by_type in self.cargos.cargo:
+                    if cargo_by_type.tipo_cargo == tipo_cargo:
+                        for termino_pot in cargo_by_type.termino_cargo:
+                            periodes += termino_pot.periodos
+                        total += cargo_by_type.importe_total
+        except AttributeError:
+            pass
+        return periodes, total
+
     def get_info_exces(self):
         """Retorna els periodes de potència"""
         periodes = []
@@ -1524,6 +2142,41 @@ class FacturaATR(Factura):
 
         return periodes, total
 
+    def get_info_generacio(self):
+        periodes = []
+        total = 0
+
+        if self.autoconsumo and self.autoconsumo.energia_excedentaria:
+            total = 0
+
+            for activa in self.autoconsumo.energia_excedentaria.terminos_energia_excedentaria:
+                periodes += activa.periodos
+
+        return periodes, total
+
+    def get_info_generacio_neta(self):
+        periodes = []
+        total = 0
+
+        if self.autoconsumo and self.autoconsumo.instalacion_gen_autoconsumo:
+            for inst in self.autoconsumo.instalacion_gen_autoconsumo:
+                for activa in inst.energia_neta_gen.terminos_energia_neta_gen:
+                    periodes += activa.periodos
+
+        return periodes, total
+
+    def get_info_autoconsumo(self):
+        periodes = []
+        total = 0
+
+        if self.autoconsumo and self.autoconsumo.instalacion_gen_autoconsumo:
+            for inst in self.autoconsumo.instalacion_gen_autoconsumo:
+                total += inst.energia_autoconsumida.importe_total
+                for activa in inst.energia_autoconsumida.terminos_energia_autoconsumida:
+                    periodes += activa.periodos
+
+        return periodes, total
+
     def get_info_reactiva(self):
         periodes = []
         total = 0
