@@ -100,6 +100,6 @@ aux = {
 
 
 from gestionatr.cli import request_p0
-distri = "catral"
+distri = "binefar"
 res = request_p0(aux[distri]['url'], aux[distri]['user'], aux[distri]['password'], aux[distri]['file'])
 print res
